{-# LANGUAGE MagicHash #-}

module Curry_Base(C_NDClass (..), C_HOClass (..)) where

import Basics
import qualified Curry_Prelude
data C_NDClass
     = C_D
     | C_ND
     | Choice_C_NDClass ID C_NDClass C_NDClass
     | Choices_C_NDClass ID ([C_NDClass])
     | Fail_C_NDClass
     | Guard_C_NDClass Constraints C_NDClass

instance Show C_NDClass where
  showsPrec d (Choice_C_NDClass i x y) = showsChoice d i x y
  showsPrec d (Choices_C_NDClass i xs) = showsChoices d i xs
  showsPrec d (Guard_C_NDClass c e) = showsGuard d c e
  showsPrec _ Fail_C_NDClass = showChar '!'
  showsPrec _ C_D = showString "D"
  showsPrec _ C_ND = showString "ND"


instance Read C_NDClass where
  readsPrec _ s = (readParen False (\r -> [ (C_D,r0) | (_,r0) <- readQualified "Base" "D" r]) s) ++ (readParen False (\r -> [ (C_ND,r0) | (_,r0) <- readQualified "Base" "ND" r]) s)


instance NonDet C_NDClass where
  choiceCons = Choice_C_NDClass
  choicesCons = Choices_C_NDClass
  failCons = Fail_C_NDClass
  guardCons = Guard_C_NDClass
  try (Choice_C_NDClass i x y) = tryChoice i x y
  try (Choices_C_NDClass i xs) = tryChoices i xs
  try Fail_C_NDClass = Fail
  try (Guard_C_NDClass c e) = Guard c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_NDClass i x y) = f i x y
  match _ f _ _ _ _ (Choices_C_NDClass i@(NarrowedID _ _) xs) = f i xs
  match _ _ f _ _ _ (Choices_C_NDClass i@(FreeID _ _) xs) = f i xs
  match _ _ _ _ _ _ (Choices_C_NDClass i@(ChoiceID _) _) = error ("Base.NDClass.match: Choices with ChoiceID " ++ (show i))
  match _ _ _ f _ _ Fail_C_NDClass = f
  match _ _ _ _ f _ (Guard_C_NDClass c e) = f c e
  match _ _ _ _ _ f x = f x


instance Generable C_NDClass where
  generate s = Choices_C_NDClass (freeID [0,0] s) [C_D,C_ND]


instance NormalForm C_NDClass where
  ($!!) cont C_D cs = cont C_D cs
  ($!!) cont C_ND cs = cont C_ND cs
  ($!!) cont (Choice_C_NDClass i x y) cs = nfChoice cont i x y cs
  ($!!) cont (Choices_C_NDClass i xs) cs = nfChoices cont i xs cs
  ($!!) cont (Guard_C_NDClass c x) cs = guardCons c ((cont $!! x) (addCs c cs))
  ($!!) _ Fail_C_NDClass _ = failCons
  ($##) cont C_D cs = cont C_D cs
  ($##) cont C_ND cs = cont C_ND cs
  ($##) cont (Choice_C_NDClass i x y) cs = gnfChoice cont i x y cs
  ($##) cont (Choices_C_NDClass i xs) cs = gnfChoices cont i xs cs
  ($##) cont (Guard_C_NDClass c x) cs = guardCons c ((cont $## x) (addCs c cs))
  ($##) _ Fail_C_NDClass _ = failCons
  ($!<) cont C_D = cont C_D
  ($!<) cont C_ND = cont C_ND
  ($!<) cont (Choice_C_NDClass i x y) = nfChoiceIO cont i x y
  ($!<) cont (Choices_C_NDClass i xs) = nfChoicesIO cont i xs
  ($!<) cont x = cont x
  searchNF _ cont C_D = cont C_D
  searchNF _ cont C_ND = cont C_ND
  searchNF _ _ x = error ("Base.NDClass.searchNF: no constructor: " ++ (show x))


instance Unifiable C_NDClass where
  (=.=) C_D C_D cs = C_Success
  (=.=) C_ND C_ND cs = C_Success
  (=.=) _ _ _ = Fail_C_Success
  (=.<=) C_D C_D cs = C_Success
  (=.<=) C_ND C_ND cs = C_Success
  (=.<=) _ _ _ = Fail_C_Success
  bind i C_D = ((i :=: (ChooseN 0 0)):(concat []))
  bind i C_ND = ((i :=: (ChooseN 1 0)):(concat []))
  bind i (Choice_C_NDClass j l r) = [(ConstraintChoice j (bind i l) (bind i r))]
  bind i (Choices_C_NDClass j@(FreeID _ _) _) = [(i :=: (BindTo j))]
  bind i (Choices_C_NDClass j@(NarrowedID _ _) xs) = [(ConstraintChoices j (map (bind i) xs))]
  bind _ (Choices_C_NDClass i@(ChoiceID _) _) = error ("Base.NDClass.bind: Choices with ChoiceID: " ++ (show i))
  bind _ Fail_C_NDClass = [Unsolvable]
  bind i (Guard_C_NDClass c e) = (getConstrList c) ++ (bind i e)
  lazyBind i C_D = [(i :=: (ChooseN 0 0))]
  lazyBind i C_ND = [(i :=: (ChooseN 1 0))]
  lazyBind i (Choice_C_NDClass j l r) = [(ConstraintChoice j (lazyBind i l) (lazyBind i r))]
  lazyBind i (Choices_C_NDClass j@(FreeID _ _) _) = [(i :=: (BindTo j))]
  lazyBind i (Choices_C_NDClass j@(NarrowedID _ _) xs) = [(ConstraintChoices j (map (lazyBind i) xs))]
  lazyBind _ (Choices_C_NDClass i@(ChoiceID _) _) = error ("Base.NDClass.lazyBind: Choices with ChoiceID: " ++ (show i))
  lazyBind _ Fail_C_NDClass = [Unsolvable]
  lazyBind i (Guard_C_NDClass c e) = (getConstrList c) ++ [(i :=: (LazyBind (lazyBind i e)))]


instance Curry_Prelude.Curry C_NDClass where
  (=?=) (Choice_C_NDClass i x y) z cs = narrow i ((x Curry_Prelude.=?= z) cs) ((y Curry_Prelude.=?= z) cs)
  (=?=) (Choices_C_NDClass i xs) y cs = narrows cs i (\x -> (x Curry_Prelude.=?= y) cs) xs
  (=?=) (Guard_C_NDClass c e) y cs = guardCons c ((e Curry_Prelude.=?= y) (addCs c cs))
  (=?=) Fail_C_NDClass _ _ = failCons
  (=?=) z (Choice_C_NDClass i x y) cs = narrow i ((z Curry_Prelude.=?= x) cs) ((z Curry_Prelude.=?= y) cs)
  (=?=) y (Choices_C_NDClass i xs) cs = narrows cs i (\x -> (y Curry_Prelude.=?= x) cs) xs
  (=?=) y (Guard_C_NDClass c e) cs = guardCons c ((y Curry_Prelude.=?= e) (addCs c cs))
  (=?=) _ Fail_C_NDClass _ = failCons
  (=?=) C_D C_D cs = Curry_Prelude.C_True
  (=?=) C_ND C_ND cs = Curry_Prelude.C_True
  (=?=) _ _ _ = Curry_Prelude.C_False
  (<?=) (Choice_C_NDClass i x y) z cs = narrow i ((x Curry_Prelude.<?= z) cs) ((y Curry_Prelude.<?= z) cs)
  (<?=) (Choices_C_NDClass i xs) y cs = narrows cs i (\x -> (x Curry_Prelude.<?= y) cs) xs
  (<?=) (Guard_C_NDClass c e) y cs = guardCons c ((e Curry_Prelude.<?= y) (addCs c cs))
  (<?=) Fail_C_NDClass _ _ = failCons
  (<?=) z (Choice_C_NDClass i x y) cs = narrow i ((z Curry_Prelude.<?= x) cs) ((z Curry_Prelude.<?= y) cs)
  (<?=) y (Choices_C_NDClass i xs) cs = narrows cs i (\x -> (y Curry_Prelude.<?= x) cs) xs
  (<?=) y (Guard_C_NDClass c e) cs = guardCons c ((y Curry_Prelude.<?= e) (addCs c cs))
  (<?=) _ Fail_C_NDClass _ = failCons
  (<?=) C_D C_D cs = Curry_Prelude.C_True
  (<?=) C_D C_ND _ = Curry_Prelude.C_True
  (<?=) C_ND C_ND cs = Curry_Prelude.C_True
  (<?=) _ _ _ = Curry_Prelude.C_False


data C_HOClass
     = C_FO
     | C_HO
     | Choice_C_HOClass ID C_HOClass C_HOClass
     | Choices_C_HOClass ID ([C_HOClass])
     | Fail_C_HOClass
     | Guard_C_HOClass Constraints C_HOClass

instance Show C_HOClass where
  showsPrec d (Choice_C_HOClass i x y) = showsChoice d i x y
  showsPrec d (Choices_C_HOClass i xs) = showsChoices d i xs
  showsPrec d (Guard_C_HOClass c e) = showsGuard d c e
  showsPrec _ Fail_C_HOClass = showChar '!'
  showsPrec _ C_FO = showString "FO"
  showsPrec _ C_HO = showString "HO"


instance Read C_HOClass where
  readsPrec _ s = (readParen False (\r -> [ (C_FO,r0) | (_,r0) <- readQualified "Base" "FO" r]) s) ++ (readParen False (\r -> [ (C_HO,r0) | (_,r0) <- readQualified "Base" "HO" r]) s)


instance NonDet C_HOClass where
  choiceCons = Choice_C_HOClass
  choicesCons = Choices_C_HOClass
  failCons = Fail_C_HOClass
  guardCons = Guard_C_HOClass
  try (Choice_C_HOClass i x y) = tryChoice i x y
  try (Choices_C_HOClass i xs) = tryChoices i xs
  try Fail_C_HOClass = Fail
  try (Guard_C_HOClass c e) = Guard c e
  try x = Val x
  match f _ _ _ _ _ (Choice_C_HOClass i x y) = f i x y
  match _ f _ _ _ _ (Choices_C_HOClass i@(NarrowedID _ _) xs) = f i xs
  match _ _ f _ _ _ (Choices_C_HOClass i@(FreeID _ _) xs) = f i xs
  match _ _ _ _ _ _ (Choices_C_HOClass i@(ChoiceID _) _) = error ("Base.HOClass.match: Choices with ChoiceID " ++ (show i))
  match _ _ _ f _ _ Fail_C_HOClass = f
  match _ _ _ _ f _ (Guard_C_HOClass c e) = f c e
  match _ _ _ _ _ f x = f x


instance Generable C_HOClass where
  generate s = Choices_C_HOClass (freeID [0,0] s) [C_FO,C_HO]


instance NormalForm C_HOClass where
  ($!!) cont C_FO cs = cont C_FO cs
  ($!!) cont C_HO cs = cont C_HO cs
  ($!!) cont (Choice_C_HOClass i x y) cs = nfChoice cont i x y cs
  ($!!) cont (Choices_C_HOClass i xs) cs = nfChoices cont i xs cs
  ($!!) cont (Guard_C_HOClass c x) cs = guardCons c ((cont $!! x) (addCs c cs))
  ($!!) _ Fail_C_HOClass _ = failCons
  ($##) cont C_FO cs = cont C_FO cs
  ($##) cont C_HO cs = cont C_HO cs
  ($##) cont (Choice_C_HOClass i x y) cs = gnfChoice cont i x y cs
  ($##) cont (Choices_C_HOClass i xs) cs = gnfChoices cont i xs cs
  ($##) cont (Guard_C_HOClass c x) cs = guardCons c ((cont $## x) (addCs c cs))
  ($##) _ Fail_C_HOClass _ = failCons
  ($!<) cont C_FO = cont C_FO
  ($!<) cont C_HO = cont C_HO
  ($!<) cont (Choice_C_HOClass i x y) = nfChoiceIO cont i x y
  ($!<) cont (Choices_C_HOClass i xs) = nfChoicesIO cont i xs
  ($!<) cont x = cont x
  searchNF _ cont C_FO = cont C_FO
  searchNF _ cont C_HO = cont C_HO
  searchNF _ _ x = error ("Base.HOClass.searchNF: no constructor: " ++ (show x))


instance Unifiable C_HOClass where
  (=.=) C_FO C_FO cs = C_Success
  (=.=) C_HO C_HO cs = C_Success
  (=.=) _ _ _ = Fail_C_Success
  (=.<=) C_FO C_FO cs = C_Success
  (=.<=) C_HO C_HO cs = C_Success
  (=.<=) _ _ _ = Fail_C_Success
  bind i C_FO = ((i :=: (ChooseN 0 0)):(concat []))
  bind i C_HO = ((i :=: (ChooseN 1 0)):(concat []))
  bind i (Choice_C_HOClass j l r) = [(ConstraintChoice j (bind i l) (bind i r))]
  bind i (Choices_C_HOClass j@(FreeID _ _) _) = [(i :=: (BindTo j))]
  bind i (Choices_C_HOClass j@(NarrowedID _ _) xs) = [(ConstraintChoices j (map (bind i) xs))]
  bind _ (Choices_C_HOClass i@(ChoiceID _) _) = error ("Base.HOClass.bind: Choices with ChoiceID: " ++ (show i))
  bind _ Fail_C_HOClass = [Unsolvable]
  bind i (Guard_C_HOClass c e) = (getConstrList c) ++ (bind i e)
  lazyBind i C_FO = [(i :=: (ChooseN 0 0))]
  lazyBind i C_HO = [(i :=: (ChooseN 1 0))]
  lazyBind i (Choice_C_HOClass j l r) = [(ConstraintChoice j (lazyBind i l) (lazyBind i r))]
  lazyBind i (Choices_C_HOClass j@(FreeID _ _) _) = [(i :=: (BindTo j))]
  lazyBind i (Choices_C_HOClass j@(NarrowedID _ _) xs) = [(ConstraintChoices j (map (lazyBind i) xs))]
  lazyBind _ (Choices_C_HOClass i@(ChoiceID _) _) = error ("Base.HOClass.lazyBind: Choices with ChoiceID: " ++ (show i))
  lazyBind _ Fail_C_HOClass = [Unsolvable]
  lazyBind i (Guard_C_HOClass c e) = (getConstrList c) ++ [(i :=: (LazyBind (lazyBind i e)))]


instance Curry_Prelude.Curry C_HOClass where
  (=?=) (Choice_C_HOClass i x y) z cs = narrow i ((x Curry_Prelude.=?= z) cs) ((y Curry_Prelude.=?= z) cs)
  (=?=) (Choices_C_HOClass i xs) y cs = narrows cs i (\x -> (x Curry_Prelude.=?= y) cs) xs
  (=?=) (Guard_C_HOClass c e) y cs = guardCons c ((e Curry_Prelude.=?= y) (addCs c cs))
  (=?=) Fail_C_HOClass _ _ = failCons
  (=?=) z (Choice_C_HOClass i x y) cs = narrow i ((z Curry_Prelude.=?= x) cs) ((z Curry_Prelude.=?= y) cs)
  (=?=) y (Choices_C_HOClass i xs) cs = narrows cs i (\x -> (y Curry_Prelude.=?= x) cs) xs
  (=?=) y (Guard_C_HOClass c e) cs = guardCons c ((y Curry_Prelude.=?= e) (addCs c cs))
  (=?=) _ Fail_C_HOClass _ = failCons
  (=?=) C_FO C_FO cs = Curry_Prelude.C_True
  (=?=) C_HO C_HO cs = Curry_Prelude.C_True
  (=?=) _ _ _ = Curry_Prelude.C_False
  (<?=) (Choice_C_HOClass i x y) z cs = narrow i ((x Curry_Prelude.<?= z) cs) ((y Curry_Prelude.<?= z) cs)
  (<?=) (Choices_C_HOClass i xs) y cs = narrows cs i (\x -> (x Curry_Prelude.<?= y) cs) xs
  (<?=) (Guard_C_HOClass c e) y cs = guardCons c ((e Curry_Prelude.<?= y) (addCs c cs))
  (<?=) Fail_C_HOClass _ _ = failCons
  (<?=) z (Choice_C_HOClass i x y) cs = narrow i ((z Curry_Prelude.<?= x) cs) ((z Curry_Prelude.<?= y) cs)
  (<?=) y (Choices_C_HOClass i xs) cs = narrows cs i (\x -> (y Curry_Prelude.<?= x) cs) xs
  (<?=) y (Guard_C_HOClass c e) cs = guardCons c ((y Curry_Prelude.<?= e) (addCs c cs))
  (<?=) _ Fail_C_HOClass _ = failCons
  (<?=) C_FO C_FO cs = Curry_Prelude.C_True
  (<?=) C_FO C_HO _ = Curry_Prelude.C_True
  (<?=) C_HO C_HO cs = Curry_Prelude.C_True
  (<?=) _ _ _ = Curry_Prelude.C_False

